@@ -9,6 +9,7 @@ use git2::Repository;
 use ratatui::style::Style;
 use ratatui::text::Line;
 use ratatui::text::Span;
+use regex::Regex;
 use std::borrow::Cow;
 use std::iter;
 use std::path::PathBuf;
@@ -157,6 +158,7 @@ pub(crate) fn log(
     repo: &Repository,
     limit: usize,
     rev: Option<Oid>,
+    msg_regex: Option<Regex>,
 ) -> Res<Vec<Item>> {
     let style = &config.style;
     let mut revwalk = repo.revwalk()?;
@@ -192,8 +194,8 @@ pub(crate) fn log(
         )
         .collect::<Vec<(Commit, Span)>>();
 
-    Ok(revwalk
-        .map(|oid_result| -> Res<Item> {
+    let items: Vec<Item> = revwalk
+        .map(|oid_result| -> Res<Option<Item>> {
             let oid = oid_result?;
             let commit = repo.find_commit(oid)?;
             let short_id = commit.as_object().short_id()?.as_str().unwrap().to_string();
@@ -211,24 +213,39 @@ pub(crate) fn log(
             )
             .collect::<Vec<_>>();
 
-            Ok(Item {
+            if let Some(re) = &msg_regex {
+                if !re.is_match(commit.message().unwrap_or("")) {
+                    return Ok(None);
+                }
+            }
+
+            Ok(Some(Item {
                 id: oid.to_string().into(),
                 display: Line::from(spans),
                 depth: 1,
                 target_data: Some(TargetData::Commit(oid.to_string())),
                 ..Default::default()
-            })
+            }))
         })
-        .map(|result| match result {
+        .filter_map(|result| match result {
             Ok(item) => item,
-            Err(err) => Item {
+            Err(err) => Some(Item {
                 id: err.to_string().into(),
                 display: err.to_string().into(),
                 ..Default::default()
-            },
+            }),
         })
         .take(limit)
-        .collect())
+        .collect();
+
+    if items.is_empty() {
+        Ok(vec![Item {
+            display: Line::raw("No commits found"),
+            ..Default::default()
+        }])
+    } else {
+        Ok(items)
+    }
 }
 
 pub(crate) fn blank_line() -> Item {
