@@ -1,10 +1,11 @@
-use super::{Action, OpTrait};
+use super::{set_prompt, Action, OpTrait};
 use crate::{
     items::TargetData,
-    menu::{arg::Arg, PendingMenu},
+    menu::PendingMenu,
     screen::NavMode,
     state::{root_menu, State},
     term::Term,
+    Res,
 };
 use derive_more::Display;
 use std::rc::Rc;
@@ -41,7 +42,7 @@ impl OpTrait for Quit {
                     }
                 }
             } else {
-                state.pending_menu = root_menu(&state.config).map(PendingMenu::init);
+                state.close_menu();
                 return Ok(());
             }
 
@@ -68,7 +69,10 @@ impl OpTrait for OpenMenu {
 pub(crate) struct Refresh;
 impl OpTrait for Refresh {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
-        Some(Rc::new(|state, _term| state.screen_mut().update()))
+        Some(Rc::new(|state, _term| {
+            state.close_menu();
+            state.screen_mut().update()
+        }))
     }
 }
 
@@ -77,22 +81,64 @@ impl OpTrait for Refresh {
 pub(crate) struct ToggleArg(pub String);
 impl OpTrait for ToggleArg {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
-        let arg = self.0.clone();
+        let arg_name = self.0.clone();
         Some(Rc::new(move |state, _term| {
-            if let Some(menu) = &mut state.pending_menu {
-                menu.args.entry(arg.clone().into()).and_modify(Arg::toggle);
+            let mut need_prompt = None;
+            let mut default = None;
+
+            let maybe_entry = if let Some(menu) = &mut state.pending_menu {
+                Some(menu.args.entry(arg_name.clone().into()))
+            } else {
+                None
+            };
+
+            if let Some(entry) = maybe_entry {
+                entry.and_modify(|arg| {
+                    if arg.is_active() {
+                        arg.unset();
+                    } else if arg.expects_value() {
+                        default = arg.default_as_string();
+                        need_prompt = Some(arg.display);
+                    } else {
+                        arg.set("").expect("Should succeed");
+                    }
+                });
             }
+
+            if let Some(display) = need_prompt {
+                set_prompt(
+                    state,
+                    display,
+                    parse_and_set_arg,
+                    Box::new(move |_| default.clone()),
+                    arg_name.clone(),
+                    false,
+                );
+            }
+
             Ok(())
         }))
     }
 }
 
+fn parse_and_set_arg(state: &mut State, _term: &mut Term, value: &str, arg: &String) -> Res<()> {
+    let key: &str = arg;
+    if let Some(menu) = &mut state.pending_menu {
+        if let Some(entry) = menu.args.get_mut(key) {
+            return entry.set(value);
+        }
+    }
+
+    Ok(())
+}
+
 #[derive(Display)]
 #[display(fmt = "Toggle section")]
 pub(crate) struct ToggleSection;
 impl OpTrait for ToggleSection {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
         Some(Rc::new(|state, _term| {
+            state.close_menu();
             state.screen_mut().toggle_section();
             Ok(())
         }))
@@ -105,6 +151,7 @@ pub(crate) struct MoveUp;
 impl OpTrait for MoveUp {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
         Some(Rc::new(|state, _term| {
+            state.close_menu();
             state.screen_mut().select_previous(NavMode::Normal);
             Ok(())
         }))
@@ -117,6 +164,7 @@ pub(crate) struct MoveDown;
 impl OpTrait for MoveDown {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
         Some(Rc::new(|state, _term| {
+            state.close_menu();
             state.screen_mut().select_next(NavMode::Normal);
             Ok(())
         }))
@@ -129,6 +177,7 @@ pub(crate) struct MoveDownLine;
 impl OpTrait for MoveDownLine {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
         Some(Rc::new(|state, _term| {
+            state.close_menu();
             state.screen_mut().select_next(NavMode::IncludeHunkLines);
             Ok(())
         }))
@@ -141,6 +190,7 @@ pub(crate) struct MoveUpLine;
 impl OpTrait for MoveUpLine {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
         Some(Rc::new(|state, _term| {
+            state.close_menu();
             state
                 .screen_mut()
                 .select_previous(NavMode::IncludeHunkLines);
@@ -155,6 +205,7 @@ pub(crate) struct MoveNextSection;
 impl OpTrait for MoveNextSection {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
         Some(Rc::new(|state, _term| {
+            state.close_menu();
             let depth = state.screen().get_selected_item().depth;
             state.screen_mut().select_next(NavMode::Siblings { depth });
             Ok(())
@@ -168,6 +219,7 @@ pub(crate) struct MovePrevSection;
 impl OpTrait for MovePrevSection {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
         Some(Rc::new(|state, _term| {
+            state.close_menu();
             let depth = state.screen().get_selected_item().depth;
             state
                 .screen_mut()
@@ -183,6 +235,7 @@ pub(crate) struct MoveParentSection;
 impl OpTrait for MoveParentSection {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
         Some(Rc::new(|state, _term| {
+            state.close_menu();
             let depth = state.screen().get_selected_item().depth.saturating_sub(1);
             state
                 .screen_mut()
@@ -198,6 +251,7 @@ pub(crate) struct HalfPageUp;
 impl OpTrait for HalfPageUp {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
         Some(Rc::new(|state, _term| {
+            state.close_menu();
             state.screen_mut().scroll_half_page_up();
             Ok(())
         }))
@@ -210,6 +264,7 @@ pub(crate) struct HalfPageDown;
 impl OpTrait for HalfPageDown {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
         Some(Rc::new(|state, _term| {
+            state.close_menu();
             state.screen_mut().scroll_half_page_down();
             Ok(())
         }))
