@@ -1,9 +1,11 @@
 use super::{create_prompt, Action, OpTrait};
 use crate::{items::TargetData, menu::arg::Arg, state::State, term::Term, Res};
 use derive_more::Display;
-use std::{ffi::OsString, process::Command, rc::Rc};
+use std::{process::Command, rc::Rc};
 
-pub(crate) const ARGS: &[Arg] = &[Arg::new("--rebase", "Rebase local commits", false)];
+pub(crate) fn init_args() -> Vec<Arg> {
+    vec![Arg::new_flag("--rebase", "Rebase local commits", false)]
+}
 
 #[derive(Display)]
 #[display(fmt = "Pull")]
@@ -15,6 +17,7 @@ impl OpTrait for Pull {
             cmd.arg("pull");
             cmd.args(state.pending_menu.as_ref().unwrap().args());
 
+            state.close_menu();
             state.run_cmd_async(term, &[], cmd)?;
             Ok(())
         }))
@@ -26,16 +29,17 @@ impl OpTrait for Pull {
 pub(crate) struct PullElsewhere;
 impl OpTrait for PullElsewhere {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
-        Some(create_prompt("Select remote", pull_elsewhere))
+        Some(create_prompt("Select remote", pull_elsewhere, true))
     }
 }
 
-fn pull_elsewhere(state: &mut State, term: &mut Term, args: &[OsString], remote: &str) -> Res<()> {
+fn pull_elsewhere(state: &mut State, term: &mut Term, remote: &str) -> Res<()> {
     let mut cmd = Command::new("git");
     cmd.args(["pull"]);
-    cmd.args(args);
+    cmd.args(state.pending_menu.as_ref().unwrap().args());
     cmd.arg(remote);
 
+    state.close_menu();
     state.run_cmd_async(term, &[], cmd)?;
     Ok(())
 }
