@@ -2,30 +2,33 @@ use super::{create_prompt, create_prompt_with_default, Action, OpTrait};
 use crate::{items::TargetData, menu::arg::Arg, state::State, term::Term, Res};
 use derive_more::Display;
 use git2::{Repository, Status, StatusOptions};
-use std::{ffi::OsString, process::Command, rc::Rc};
+use std::{process::Command, rc::Rc};
 
-pub(crate) const ARGS: &[Arg] = &[
-    Arg::new("--include-untracked", "Also save untracked files", true),
-    Arg::new("--all", "Also save untracked and ignored files", false),
-];
+pub(crate) fn init_args() -> Vec<Arg> {
+    vec![
+        Arg::new_flag("--include-untracked", "Also save untracked files", true),
+        Arg::new_flag("--all", "Also save untracked and ignored files", false),
+    ]
+}
 
 #[derive(Display)]
 #[display(fmt = "Stash")]
 pub(crate) struct Stash;
 impl OpTrait for Stash {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
-        Some(create_prompt("Stash message", stash_push))
+        Some(create_prompt("Stash message", stash_push, true))
     }
 }
 
-fn stash_push(state: &mut State, term: &mut Term, args: &[OsString], input: &str) -> Res<()> {
+fn stash_push(state: &mut State, term: &mut Term, input: &str) -> Res<()> {
     let mut cmd = Command::new("git");
     cmd.args(["stash", "push"]);
-    cmd.args(args);
+    cmd.args(state.pending_menu.as_ref().unwrap().args());
     if !input.is_empty() {
         cmd.args(["--message", input]);
     }
 
+    state.close_menu();
     state.run_cmd(term, &[], cmd)?;
     Ok(())
 }
@@ -35,16 +38,11 @@ fn stash_push(state: &mut State, term: &mut Term, args: &[OsString], input: &str
 pub(crate) struct StashIndex;
 impl OpTrait for StashIndex {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
-        Some(create_prompt("Stash message", stash_push_index))
+        Some(create_prompt("Stash message", stash_push_index, true))
     }
 }
 
-fn stash_push_index(
-    state: &mut State,
-    term: &mut Term,
-    _args: &[OsString],
-    input: &str,
-) -> Res<()> {
+fn stash_push_index(state: &mut State, term: &mut Term, input: &str) -> Res<()> {
     let mut cmd = Command::new("git");
     // --all / --unclude-untracked are not allowed together with --staged
     cmd.args(["stash", "push", "--staged"]);
@@ -52,6 +50,7 @@ fn stash_push_index(
         cmd.args(["--message", input]);
     }
 
+    state.close_menu();
     state.run_cmd(term, &[], cmd)?;
     Ok(())
 }
@@ -63,10 +62,11 @@ impl OpTrait for StashWorktree {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
         Some(Rc::new(|state: &mut State, term: &mut Term| -> Res<()> {
             if is_working_tree_empty(&state.repo)? {
+                state.close_menu();
                 return Err("Cannot stash: working tree is empty".into());
             }
 
-            let mut create_prompt = create_prompt("Stash message", stash_worktree);
+            let mut create_prompt = create_prompt("Stash message", stash_worktree, true);
             Rc::get_mut(&mut create_prompt).unwrap()(state, term)?;
             Ok(())
         }))
@@ -93,20 +93,23 @@ fn is_working_tree_empty(repo: &Repository) -> Res<bool> {
     Ok(!is_working_tree_not_empty)
 }
 
-fn stash_worktree(state: &mut State, term: &mut Term, args: &[OsString], input: &str) -> Res<()> {
+fn stash_worktree(state: &mut State, term: &mut Term, input: &str) -> Res<()> {
     let need_to_stash_index = is_something_staged(&state.repo)?;
 
+    let mut cmd = Command::new("git");
+    cmd.args(["stash", "push"]);
+    cmd.args(state.pending_menu.as_ref().unwrap().args());
+
+    state.close_menu();
+
     if need_to_stash_index {
         let mut cmd = Command::new("git");
         cmd.args(["stash", "push", "--staged"]);
         state.run_cmd(term, &[], cmd)?;
     }
 
-    let mut cmd = Command::new("git");
-    cmd.args(["stash", "push"]);
-    cmd.args(args);
     if !input.is_empty() {
-        cmd.args(["--message", &input]);
+        cmd.args(["--message", input]);
     }
     state.run_cmd(term, &[], cmd)?;
 
@@ -142,23 +145,19 @@ fn is_something_staged(repo: &Repository) -> Res<bool> {
 pub(crate) struct StashKeepIndex;
 impl OpTrait for StashKeepIndex {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
-        Some(create_prompt("Stash message", stash_push_keep_index))
+        Some(create_prompt("Stash message", stash_push_keep_index, true))
     }
 }
 
-fn stash_push_keep_index(
-    state: &mut State,
-    term: &mut Term,
-    args: &[OsString],
-    input: &str,
-) -> Res<()> {
+fn stash_push_keep_index(state: &mut State, term: &mut Term, input: &str) -> Res<()> {
     let mut cmd = Command::new("git");
     cmd.args(["stash", "push", "--keep-index"]);
-    cmd.args(args);
+    cmd.args(state.pending_menu.as_ref().unwrap().args());
     if !input.is_empty() {
         cmd.args(["--message", input]);
     }
 
+    state.close_menu();
     state.run_cmd(term, &[], cmd)?;
     Ok(())
 }
@@ -172,14 +171,17 @@ impl OpTrait for StashPop {
             "Pop stash",
             stash_pop,
             selected_stash,
+            true,
         ))
     }
 }
 
-fn stash_pop(state: &mut State, term: &mut Term, _args: &[OsString], input: &str) -> Res<()> {
+fn stash_pop(state: &mut State, term: &mut Term, input: &str) -> Res<()> {
     let mut cmd = Command::new("git");
     cmd.args(["stash", "pop", "-q"]);
     cmd.arg(input);
+
+    state.close_menu();
     state.run_cmd(term, &[], cmd)?;
     Ok(())
 }
@@ -193,14 +195,17 @@ impl OpTrait for StashApply {
             "Apply stash",
             stash_apply,
             selected_stash,
+            true,
         ))
     }
 }
 
-fn stash_apply(state: &mut State, term: &mut Term, _args: &[OsString], input: &str) -> Res<()> {
+fn stash_apply(state: &mut State, term: &mut Term, input: &str) -> Res<()> {
     let mut cmd = Command::new("git");
     cmd.args(["stash", "apply", "-q"]);
     cmd.arg(input);
+
+    state.close_menu();
     state.run_cmd(term, &[], cmd)?;
     Ok(())
 }
@@ -214,14 +219,17 @@ impl OpTrait for StashDrop {
             "Drop stash",
             stash_drop,
             selected_stash,
+            true,
         ))
     }
 }
 
-fn stash_drop(state: &mut State, term: &mut Term, _args: &[OsString], input: &str) -> Res<()> {
+fn stash_drop(state: &mut State, term: &mut Term, input: &str) -> Res<()> {
     let mut cmd = Command::new("git");
     cmd.args(["stash", "drop"]);
     cmd.arg(input);
+
+    state.close_menu();
     state.run_cmd(term, &[], cmd)?;
     Ok(())
 }
